CREATE TABLE PUBLIC.ORGANIZATION (
	ID INTEGER NOT NULL,
	UNIQUE_NAME VARCHAR(32) NOT NULL,
	NAME VARCHAR(256),
	DESCRIPTION VARCHAR(512),
	PRIMARY KEY (ID)
);

CREATE TABLE PUBLIC.ROLE (
	ID INTEGER NOT NULL,
	ROLE_NAME VARCHAR(256) NOT NULL,
	DESCRIPTION VARCHAR(256),
	PRIMARY KEY (ID)
);

CREATE TABLE PUBLIC.USER_ACCOUNT (
	ID DECIMAL(65535,32767) NOT NULL,
	USER_NAME VARCHAR(32) NOT NULL,
	EMAIL VARCHAR(256),
	PASSWORD_SALT VARCHAR(128) NOT NULL,
	PASSWORD VARCHAR(128) NOT NULL,
	PASSWORD_QUESTION VARCHAR(256) NOT NULL,
	PASSWORD_ANSWER VARCHAR(128) NOT NULL,
	ENABLED BOOLEAN NOT NULL,
	LOCKED BOOLEAN NOT NULL,
	LAST_LOCK_OUT_DATE TIMESTAMP DEFAULT TIMESTAMP '0-01-01 00:00:00.0',
	CREATED_DATE TIMESTAMP DEFAULT CURRENT_TIMESTAMP() NOT NULL,
	LAST_LOGIN_DATE TIMESTAMP DEFAULT CURRENT_TIMESTAMP() NOT NULL,
	LAST_PASSWORD_CHANGED_DATE TIMESTAMP DEFAULT CURRENT_TIMESTAMP() NOT NULL,
	FAILED_PWD_WIN_START TIMESTAMP DEFAULT TIMESTAMP '0-01-01 00:00:00.0',
	FAILED_PWD_ANSWER_WIN_START TIMESTAMP DEFAULT TIMESTAMP '0-01-01 00:00:00.0',
	COMMENTS CLOB(2147483647),
	PRIMARY KEY (ID)
);

CREATE TABLE PUBLIC.USER_DETAILS (
	USER_ID DECIMAL(65535,32767) NOT NULL,
	LAST_NAME VARCHAR(64) NOT NULL,
	FIRST_NAME VARCHAR(64) NOT NULL,
	MIDDLE_NAME VARCHAR(25),
	EMPLOYEE_ID VARCHAR(32) NOT NULL,
	UNIQUE_NAME VARCHAR(32),
	INSTITUTION_NAME VARCHAR(128) NOT NULL,
	DEPARTMENT_NAME VARCHAR(128),
	PHONE VARCHAR(32),
	PAGER VARCHAR(16),
	PRIMARY KEY (USER_ID)
);

CREATE TABLE PUBLIC.USER_ORGANIZATION (
	ID DECIMAL(65535,32767) NOT NULL,
	USER_ID DECIMAL(65535,32767) NOT NULL,
	ORGANIZATION_ID DECIMAL(65535,32767) NOT NULL,
	APPROVED BOOLEAN DEFAULT FALSE NOT NULL,
	PRIMARY KEY (ID)
);

CREATE TABLE PUBLIC.USER_ROLE (
	USER_ID INTEGER NOT NULL,
	ROLE_ID INTEGER NOT NULL
);

ALTER TABLE PUBLIC.USER_DETAILS
	ADD FOREIGN KEY (USER_ID) 
	REFERENCES PUBLIC.USER_ACCOUNT (ID);



ALTER TABLE PUBLIC.USER_ORGANIZATION
	ADD FOREIGN KEY (ORGANIZATION_ID) 
	REFERENCES PUBLIC.ORGANIZATION (ID);

ALTER TABLE PUBLIC.USER_ORGANIZATION
	ADD FOREIGN KEY (USER_ID) 
	REFERENCES PUBLIC.USER_ACCOUNT (ID);



ALTER TABLE PUBLIC.USER_ROLE
	ADD FOREIGN KEY (ROLE_ID) 
	REFERENCES PUBLIC.ROLE (ID);

ALTER TABLE PUBLIC.USER_ROLE
	ADD FOREIGN KEY (USER_ID) 
	REFERENCES PUBLIC.USER_ACCOUNT (ID);

ALTER TABLE PUBLIC.USER_ROLE ADD PRIMARY KEY (USER_ID, ROLE_ID);

CREATE UNIQUE INDEX PUBLIC.CONSTRAINT_INDEX_D ON PUBLIC.ORGANIZATION (UNIQUE_NAME);

CREATE SEQUENCE PUBLIC.SEQ_ORGANIZATION START WITH 50 INCREMENT BY 1;
CREATE SEQUENCE PUBLIC.SEQ_ROLE START WITH 50 INCREMENT BY 1;
CREATE SEQUENCE PUBLIC.SEQ_USER_ACCOUNT START WITH 50 INCREMENT BY 1;
CREATE SEQUENCE PUBLIC.SEQ_USER_DETAILS START WITH 50 INCREMENT BY 1;
CREATE SEQUENCE PUBLIC.SEQ_USER_ORGANIZATION START WITH 50 INCREMENT BY 1;


